<?php

namespace Drp\NativeEventApp\Models;

use Drp\NativeEventApp\Models\Concerns\HasStyles;
use Drp\NativeEventApp\Models\Contracts\Stylable;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;

class Event extends Model implements Stylable
{
    use HasStyles;

    /** @var string  */
    protected $table = 'events';

    public function registration(): HasOne
    {
        return $this->hasOne(Registration::class, 'eventId');
    }

    public function pages(): HasMany
    {
        return $this->hasMany(Page::class, 'eventId');
    }

    public function agendas(): HasMany
    {
        return $this->hasMany(Agenda::class, 'eventId');
    }

    public function theme(): BelongsTo
    {
        return $this->belongsTo(Theme::class, 'themeId');
    }

    public function quizzes(): HasMany
    {
        return $this->hasMany(Quiz::class, 'eventId');
    }

    public function navigation(): HasOne
    {
        return $this->hasOne(Navigation::class, 'eventId');
    }

    public function getSocketServerUrlAttribute(): ?string
    {
        return config('native-event-app.socket_server.host');
    }
}
