<?php

namespace Drp\NativeEventApp\Templates;

use BenSampo\Enum\Enum;
use DateTime;

/**
 * @method static self DATE()
 * @method static self STRING()
 * @method static self INT()
 * @method static self BOOL()
 * @method static self OBJECT()
 * @method static self ARRAY()
 */
class DataType extends Enum
{
    const DATE = 'date';
    const STRING = 'string';
    const INT = 'int';
    const BOOL = 'bool';
    const OBJECT = 'object';
    const ARRAY = 'array';

    /**
     * @var \string[][]
     */
    private static $functions = [
        self::DATE => ['format'],
        self::STRING => [],
        self::INT => [],
        self::BOOL => [],
        self::OBJECT => [],
        self::ARRAY => [],
    ];

    /**
     * @param mixed $value
     */
    public static function calculate($value): self
    {
        if (is_a($value, DateTime::class)) {
            return self::DATE();
        }

        if (is_string($value)) {
            return self::STRING();
        }

        if (is_int($value)) {
            return self::INT();
        }

        if (is_array($value)) {
            return self::ARRAY();
        }

        if (is_object($value)) {
            return self::OBJECT();
        }

        if (is_bool($value)) {
            return self::BOOL();
        }

        return self::OBJECT();
    }

    public static function fromClassName(string $type): self
    {
        if (is_a($type, DateTime::class, true)) {
            return self::DATE();
        }

        if (in_array($type, self::getValues())) {
            return self::coerce($type);
        }

        return self::OBJECT();
    }

    public function canHandleMethod(string $function): bool
    {
        return in_array($function, self::$functions[$this->value]);
    }
}
