<?php

namespace Drp\NativeEventApp\Test\Support\Builders;

use Drp\NativeEventApp\Models\Event;
use Drp\NativeEventApp\Models\Quiz;

class QuizBuilder
{
    /** @var \Drp\NativeEventApp\Models\QuizQuestion[] */
    protected $questions;

    /** @var \Drp\NativeEventApp\Models\Event */
    protected $event;

    /** @var \Drp\NativeEventApp\Test\Support\Builders\QuestionBuilder */
    private $questionBuilder;

    public function __construct(QuestionBuilder $questionBuilder)
    {
        $this->questionBuilder = $questionBuilder;
    }

    public function forEvent(Event $event): self
    {
        $this->event = $event;

        return $this;
    }

    /**
     * @param int|array $questions
     */
    public function withQuestions($questions = 2): self
    {
        $this->questions = $questions;

        if (is_array($questions)) {
            $this->questions = $this->questionBuilder->makeMany($questions);

            return $this;
        }

        if (is_int($questions)) {
            $this->questions = $this->questionBuilder->make($questions);

            return $this;
        }

        if (is_callable($questions)) {
            $this->questions = $questions($this->questionBuilder);

            return $this;
        }

        return $this;
    }

    public function create(array $attributes = [])
    {
        if ($this->event) {
            $attributes = array_merge([
                'eventId' => $this->event->id,
            ], $attributes);
        }

        return tap(factory(Quiz::class)->create($attributes), function (Quiz $quiz) {
            $questions = $this->questions;

            foreach ($questions as $question) {
                $question->quiz()->associate($quiz);
                $question->save();

                $this->questionBuilder->saveAnswers($question);
            }
        });
    }
}
