<?php

namespace Drp\NativeEventApp\Test\Unit;

use Drp\NativeEventApp\Models\Quiz;
use Drp\NativeEventApp\Models\QuizAnswer;
use Drp\NativeEventApp\Models\QuizQuestion;
use Drp\NativeEventApp\Quiz\ApiQuizOperator;
use Drp\NativeEventApp\Quiz\Client\QuizClient;
use Drp\NativeEventApp\Test\TestCase;
use Mockery;

class ApiQuizOperatorTest extends TestCase
{
    /** @test */
    public function can_create_quiz()
    {
        $quiz = factory(Quiz::class)->make();

        $mockClient = Mockery::spy(QuizClient::class);
        $apiOperator = new ApiQuizOperator($mockClient);

        $apiOperator->create($quiz);

        $mockClient->shouldHaveReceived('post')->with('', [
            'id' => $quiz->id,
            'question' => null,
        ])->once();
    }

    /** @test */
    public function can_start_quiz()
    {
        $quiz = factory(Quiz::class)->make();

        $mockClient = Mockery::spy(QuizClient::class);
        $apiOperator = new ApiQuizOperator($mockClient);

        $apiOperator->start($quiz);

        $mockClient->shouldHaveReceived('post')->with($quiz->id . '/start')->once();
    }

    /** @test */
    public function can_end_quiz()
    {
        $quiz = factory(Quiz::class)->make();

        $mockClient = Mockery::spy(QuizClient::class);
        $apiOperator = new ApiQuizOperator($mockClient);

        $apiOperator->end($quiz);

        $mockClient->shouldHaveReceived('post')->with($quiz->id . '/end')->once();
    }

    /** @test */
    public function can_delete_quiz()
    {
        $quiz = factory(Quiz::class)->make(['id' => 1]);

        $mockClient = Mockery::spy(QuizClient::class);
        $apiOperator = new ApiQuizOperator($mockClient);

        $apiOperator->delete($quiz);

        $mockClient->shouldHaveReceived('delete')->with($quiz->id)->once();
    }

    /** @test */
    public function can_start_question()
    {
        $question = factory(QuizQuestion::class)->make();

        $mockClient = Mockery::spy(QuizClient::class);
        $apiOperator = new ApiQuizOperator($mockClient);

        $apiOperator->startQuestion($question->quiz, $question);

        $mockClient->shouldHaveReceived('post')->with($question->quiz->id . '/question/' . $question->id . '/start', [
            'question' => $question->id,
        ])->once();
    }

    /** @test */
    public function can_end_question()
    {
        $question = factory(QuizQuestion::class)->create();
        factory(QuizAnswer::class)->create(['quizQuestionId' => $question->id, 'isCorrect' => false]);
        factory(QuizAnswer::class)->create(['quizQuestionId' => $question->id, 'isCorrect' => true]);

        $mockClient = Mockery::spy(QuizClient::class);
        $apiOperator = new ApiQuizOperator($mockClient);

        $apiOperator->endQuestion($question->quiz, $question);

        $mockClient->shouldHaveReceived('post')
            ->with($question->quiz->id . '/question/' . $question->id . '/end')
            ->once();
    }

    /** @test */
    public function can_reveal_answer()
    {
        $question = factory(QuizQuestion::class)->create();
        $wrongAnswer = factory(QuizAnswer::class)->create(['quizQuestionId' => $question->id, 'isCorrect' => false]);
        $correctAnswer = factory(QuizAnswer::class)->create(['quizQuestionId' => $question->id, 'isCorrect' => true]);

        $mockClient = Mockery::spy(QuizClient::class);
        $apiOperator = new ApiQuizOperator($mockClient);

        $apiOperator->revealAnswer($question->quiz, $question);

        $mockClient->shouldHaveReceived('post')->with($question->quiz->id . '/question/' . $question->id . '/end', [
            'answer' => $correctAnswer->id,
        ])->once();
    }
}
