<?php

namespace Drp\NativeEventApp\Test\Unit\Modules;

use Drp\NativeEventApp\Modules\Modules;
use Drp\NativeEventApp\Modules\Types\Form\Form;
use Drp\NativeEventApp\Modules\Types\Form\Select;
use Drp\NativeEventApp\Modules\Types\Form\TextBox;
use Drp\NativeEventApp\Modules\Types\Text;
use Drp\NativeEventApp\Test\TestCase;

class FormTest extends TestCase
{
    /** @test */
    public function form_can_be_serialized(): void
    {
        $form = Form::make('form_1')
            ->children(Modules::make([
                TextBox::make('input')
                    ->label('Answer me this')
                    ->placeholder('Enter input')
                    ->value('Default value'),
                TextBox::make('required-input')
                    ->label('You must answer this')
                    ->placeholder('Enter input')
                    ->value('Default value')
                    ->required(),
                Text::make('My test'),
                Select::make('select')
                    ->label('Select dropdown')
                    ->options([
                        'Label One' => '1',
                        'Label Two' => '2',
                    ])
                    ->placeholder('Please select')
                    ->value('1'),
            ]));

        $this->assertEquals(json_encode([
            'type' => 'Form',
            'identifier' => 'form_1',
            'children' => [
                [
                    'type' => 'FormTextBox',
                    'label' => 'Answer me this',
                    'value' => 'Default value',
                    'name' => 'input',
                    'required' => false,
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                    'placeholder' => 'Enter input',
                ],
                [
                    'type' => 'FormTextBox',
                    'label' => 'You must answer this',
                    'value' => 'Default value',
                    'name' => 'required-input',
                    'required' => true,
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                    'placeholder' => 'Enter input',
                ],
                [
                    'type' => 'Text',
                    'text' => 'My test',
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                ],
                [
                    'type' => 'FormSelect',
                    'options' => [
                        ['label' => 'Label One', 'value' => '1'],
                        ['label' => 'Label Two', 'value' => '2'],
                    ],
                    'label' => 'Select dropdown',
                    'value' => '1',
                    'name' => 'select',
                    'required' => false,
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                    'placeholder' => 'Please select',
                ],
            ],
            'styles' => [],
            'className' => [],
            'viewableBy' => [],
        ]), $form->toJson());
    }
}
